/*
 * Phillips saa7191 interface for video spigot.
 *
 * Copyright (c) 1995, Jim Lowe. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met: 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer. 2.
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Version 1.2, Apr 25, 1995.
 */

#include	<stdio.h>
#include	"spigot_saa7191.h"

static unsigned char def_regs[] = {
	/* value   Subaddress	Comment */
	0x50,	/* 0x00		increment delay */
	0x30,	/* 0x01		H sync beginning for 50 hz */
	0x00,	/* 0x02		H sync stop for 50 hz */
	0xe8,	/* 0x03		H clamping beginning for 50 hz */
	0xb6,	/* 0x04		H clamping stop for 50 hz */
	0xf4,	/* 0x05		H sync position for 50 hz */
	0x01,	/* 0x06		luminance bandwidth control */
	0x00,	/* 0x07		hue control (0 degree) */
	0xf8,	/* 0x08		colour-killer threshold QUAM */
	0xf8,	/* 0x09		colour-killer threshold SECAM */
	0x90,	/* 0x0a		PAL switch sensitivity */
	0x90,	/* 0x0b		SECAM switch sensitivity */
	0x00,	/* 0x0c		chroma gain control settings */
	0x00,	/* 0x0d		standard/mode control */
	0x78,	/* 0x0e		I/O and clock control */
	0x99,	/* 0x0f		miscellaneous control #1 */
	0x00,	/* 0x10		miscellaneous control #2 */
	0x2c,	/* 0x11		chrominance gain nominal value (NTSC-0x59PAL) */
	0x00,	/* 0x12		set to zero */
	0x00,	/* 0x13		set to zero */
	0x34,	/* 0x14		H sync beginning for 60 hz */
	0x0a,	/* 0x15		H sync stop for 60 hz */
	0xf4,	/* 0x16		H clamping beginning for 60 hz */
	0xce,	/* 0x17		H clamping stop for 60 hz */
	0xf4,	/* 0x18		H sync position for 60 hz */
} ;
static int		saa7191_num_regs = sizeof(def_regs)/sizeof(char);
static unsigned char	*saa7191_regs = def_regs;

unsigned char
saa7191_status(void)
{        
unsigned char   status;

        i2c_start();
        i2c_write(SLAVE_ADDRESS_READ);
        status = i2c_read();
        i2c_stop();

        return(status);
}

int
saa7191_init(unsigned char table[], int size)
{
int	i;
int	no_error;

	if(table == (unsigned char *)0 || size == 0) {
		table = saa7191_regs;
		size = saa7191_num_regs;
	} else {
		saa7191_regs = table;
		saa7191_num_regs = size;
	}

	no_error = 1;

#ifdef INIT_DEBUG
	for(i=0x00; i< size && no_error; i++) {
		unsigned char status;
		
		status=saa7191_status();

		i2c_start();
		no_error = i2c_write(SLAVE_ADDRESS_WRITE);
		if(no_error)	no_error = i2c_write(i);
		if(no_error)	no_error = i2c_write(table[i]);
		i2c_stop();
		if(i == 0) 
			printf("SubAddr\tValue\tBefore\tAfter\n");
		printf("0x%x\t0x%x\t0x%x\t0x%x\n", i, table[i],
			status, saa7191_status());
	}
#else
	i2c_start();

	no_error = i2c_write(SLAVE_ADDRESS_WRITE);
	if(no_error)
		no_error = i2c_write(0x00);	/* Subaddress 0 */
	for(i=0; i < size && no_error; i++)
		no_error=i2c_write(table[i]);	/* 7191 auto inc mode */
	i2c_stop();
#endif
	
	if(!no_error) 
		fprintf(stderr, 
		"saa7191_init: Couldn't write subaddress = %d, data=0x%x\n",
			i-1, table[i]);

	return no_error;
}

static	struct	name_table {
	char		*name;
	unsigned char	subaddr;
	int		min_v;
	int		max_v;
	int		scale;
	int		bit_pos;	/* 0 (LSB) to 7 (MSB) */
	unsigned char	mask;
} name_table[] = {
#define	SCALE	0
#define NOSCALE 1
/*	name	sub	min	max	scale	pos	mask (2^num bits - 1) */
{	"IDEL", 0x00,	-1024,	-4,	SCALE,	0,	0xff},
{	"HSYB", 0x01,	-382,	128,	SCALE,	0,	0xff},
{	"HSYS", 0x02,	-382,	128,	SCALE,	0,	0xff},
{	"HCLB", 0x03,	-254,	256,	SCALE,	0,	0xff},
{	"HCLS", 0x04,	-254,	256,	SCALE,	0,	0xff},
{	"HPHI", 0x05,	-936,	944,	SCALE,	0,	0xff},
{	"BYPS", 0x06,	0,	1,	1,	7,	0x01},
{	"PREF", 0x06,	0,	1,	1,	6,	0x01},
{	"BPSS", 0x06,	0,	3,	1,	4,	0x03},
{	"CORI", 0x06,	0,	3,	1,	2,	0x03},
{	"APER", 0x06,	0,	3,	1,	0,	0x03},
{	"HUE",	0x07,	-180,	179,	SCALE,	0,	0xff},
{	"CKTQ",	0x08,	-30,	-18,	SCALE,	0,	0xff},
{	"CKTS",	0x09,	-30,	-18,	SCALE,	0,	0xff},
{	"PLSE",	0x0a,	0,	0xff,	1,	0,	0xff},
{	"SESE", 0x0b,	0,	0xff,	1,	0,	0xff},
{	"COLO",	0x0c,	0,	1,	1,	7,	0x01},
{	"LFIS", 0x0c,	0,	3,	1,	5,	0x03},
{	"VTRC", 0x0d,	0,	1,	1,	7,	0x01},
{	"NFEN", 0x0d,	0,	1,	1,	3,	0x01},
{	"HRMV", 0x0d,	0,	1,	1,	2,	0x01},
{	"GPSW0",0x0d,	0,	1,	1,	1,	0x01},
{	"SECS", 0x0d,	0,	1,	1,	0,	0x01},
{	"HPLL", 0x0e,	0,	1,	1,	7,	0x01},
{	"OEDC", 0x0e,	0,	1,	1,	6,	0x01},
{	"OEHS", 0x0e,	0,	1,	1,	5,	0x01},
{	"OEVS",	0x0e,	0,	1,	1,	4,	0x01},
{	"OEDY", 0x0e,	0,	1,	1,	3,	0x01},
{	"CHRS",	0x0e,	0,	1,	1,	2,	0x01},
{	"GPSW2",0x0e,	0,	1,	1,	1,	0x01},
{	"GPSW1",0x0e,	0,	1,	1,	0,	0x01},
{	"AUFD", 0x0f,	0,	1,	1,	7,	0x01},
{	"FSEL", 0x0f,	0,	1,	1,	6,	0x01},
{	"SXCR", 0x0f,	0,	1,	1,	5,	0x01},
{	"SCEN", 0x0f,	0,	1,	1,	4,	0x01},
{	"OFTS", 0x0f,	0,	1,	1,	3,	0x01},
{	"YDEL", 0x0f,	-4,	3,	1,	0,	0x07},
{	"HRFS", 0x10,	0,	1,	1,	2,	0x01},
{	"VNOI", 0x10,	0,	3,	1,	0,	0x03},
{	"CHCV", 0x11,	0,	0xff,	1,	0,	0xff},
{	"HS6B", 0x14,	-382,	128,	SCALE,	0,	0xff},
{	"HS6S", 0x15,	-382,	128,	SCALE,	0,	0xff},
{	"HC6B", 0x16,	-254,	256,	SCALE,	0,	0xff},
{	"HC6S", 0x17,	-254,	256,	SCALE,	0,	0xff},
{	"HP6I", 0x18,	-776,	776,	SCALE,	0,	0xff},
{	"",	0x0,	0,	0,	0,	0,	0},
};

int
saa7191_setv(char *name, int value)
{
struct	name_table	*p = name_table;
int			no_error = 1;
unsigned char		v;
unsigned char		mask;

	/*
	 * Search for saa7191 register in list.
	 */
	while(*p->name && strcmp(p->name, name) != 0)
		p++;
	if(!*p->name)
		return 0;

	/*
	 * Make sure it is addressable.
	 */
	if(p->subaddr > saa7191_num_regs)
		return 0;

	/*
	 * Check for validity.
	 */
	if(value < p->min_v || value > p->max_v)
		return 0;

	/*
	 * Scale the value.
	 */
	if(p->scale == SCALE) {
		switch(p->subaddr){
		case SUB_INC_DELAY:
			value = (char)(-value/4);
			break;
		case SUB_H_SYNC_BEGIN_50_HZ:
		case SUB_H_SYNC_BEGIN_60_HZ:
		case SUB_H_SYNC_STOP_50_HZ:
		case SUB_H_SYNC_STOP_60_HZ:
			value = (char)(-value/2);
			break;
		case SUB_H_CLAMP_BEGIN_50_HZ:
		case SUB_H_CLAMP_STOP_50_HZ:
		case SUB_H_CLAMP_BEGIN_60_HZ:
		case SUB_H_CLAMP_STOP_60_HZ:
			value = (char)(-value/2);
			break;
		case SUB_H_SYNC_AFTER_PHI1_50_HZ:
		case SUB_H_SYNC_AFTER_PHI1_60_HZ:
			value = (char)(-value/8);
			break;
		case SUB_HUE_CONTROL:
			value = (char)(value*128/180);
			break;
		case SUB_COLOUR_KILLER_THRES_QAM:
		case SUB_COLOUR_KILLER_THRES_SECAM:
			value = (char)(-value - 18);
			break;
		default:
			fprintf(stderr, "saa7191 set value: can't scale 0x%x\n",
				p->subaddr);
			break;
		}
	}

	/*
	 * Set the value.
	 */
	v = saa7191_regs[p->subaddr];
	mask = p->mask;
	mask <<= p->bit_pos;

	v &= ~mask;

	value <<= p->bit_pos;
	v |= value & mask;

	i2c_start();	no_error = i2c_write(SLAVE_ADDRESS_WRITE);
	if(no_error)	no_error = i2c_write(p->subaddr);
	if(no_error)	no_error = i2c_write(v);
	i2c_stop();
	if(no_error)	saa7191_regs[p->subaddr] = v;
#ifdef SET_DEBUG
	printf("saa7191 set value as: 0x%x 0x%x\n", p->subaddr, v);
#endif

	return no_error;
}

int
saa7191_getv(char *name)
{
struct	name_table	*p = name_table;
int			v;
unsigned char		mask;

	/*
	 * Search for saa7191 register in list.
	 */
	while(*p->name && strcmp(p->name, name) != 0)
		p++;
	if(!*p->name)
		return 0;

	/*
	 * Make sure it is addressable.
	 */
	if(p->subaddr > saa7191_num_regs)
		return 0;

	v = saa7191_regs[p->subaddr];
	mask = p->mask;
	mask <<= p->bit_pos;

	v &= mask;
	v >>= p->bit_pos;

	/*
	 * scale the value.
	 */
	if(p->scale == SCALE) {
		switch(p->subaddr){
		case SUB_INC_DELAY:
			v = -(char)v*4;
			break;
		case SUB_H_SYNC_BEGIN_50_HZ:
		case SUB_H_SYNC_STOP_50_HZ:
		case SUB_H_SYNC_BEGIN_60_HZ:
		case SUB_H_SYNC_STOP_60_HZ:
			v = -(char)v*2;
			break;
		case SUB_H_CLAMP_BEGIN_50_HZ:
		case SUB_H_CLAMP_STOP_50_HZ:
		case SUB_H_CLAMP_BEGIN_60_HZ:
		case SUB_H_CLAMP_STOP_60_HZ:
			v = -(char)v*2;
			break;
		case SUB_H_SYNC_AFTER_PHI1_50_HZ:
		case SUB_H_SYNC_AFTER_PHI1_60_HZ:
			v = -(char)v*8;
			break;
		case SUB_HUE_CONTROL:
			v = (char)v*180/128;
			break;
		case SUB_COLOUR_KILLER_THRES_QAM:
		case SUB_COLOUR_KILLER_THRES_SECAM:
			v = -(char)(v+18);
			break;
		default:
			fprintf(stderr, "saa7191 get value: can't scale 0x%x\n",
				p->subaddr);
			break;
		}
	}

#ifdef GET_DEBUG
	printf("saa7191 get value: 0x%x 0x%x\n", p->subaddr, v);
#endif
	return v;
}

